/* 	libvocenabler: - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	         YU Yijun <yijun.yu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef	__LIPS_VOC_EVENT_H
#define	__LIPS_VOC_EVENT_H

#include "voctype.h"
#include "vocerror.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

/**
 *@brief Event types
 */
  typedef enum
  {

    VOC_EVT_LINE,     /**< Scope: session. A line has been added or removed. 
				 *No event specific data. */

    VOC_EVT_LINE_SIGNAL,       /**<signal strength of a line has changed, event specific data ('evt_data') is 'lid' */

    VOC_EVT_LINE_CALL,                  /**< Scope: a line. A new call has been created on the line. 
				 *Event specific data ('evt_data') is 'cid' */

    VOC_EVT_CALL_REMOTE_STATUS,      /**< an incoming call is not accepted or a connected call is remotely hungup */


    VOC_EVT_CALL_STATUS,      /**< Scope: a call. Status of the call (e.g the call state) has been changed.
				 *Event specific data ('evt_data') is 'cid' */

    VOC_EVT_EXCEPTION,      /**< Scope: session. some error happend. 
							*Event specific data ('evt_data') is error code voc_err_t */

  } voc_evt_type_t;



/**
 *@brief	Event specific data. 
 *Passed in the 'evt_data' field of the 'voc_evt_t' structure,
 *typically contains the target Line ID 'lid' or Call ID 'cid'.
 */
  typedef uint32_t voc_evt_data_t;

/* 
typedef struct {
	voc_cid_t	cid;
	Int8 *peer_number;	
} voc_evt_data_t;
*/


/**
 *@brief Event descriptor
 */

  typedef struct voc_evt voc_evt_t;

/**
 *@brief voice call callback type
 */
  typedef void (*voc_cb_t) (voc_sid_t sid, voc_evt_t *evt);

  struct voc_evt
  {
    voc_evt_type_t evt_type;            /**< Event Type */
    voc_cb_t cb;                        /**< Callback */
    void *user_data;                    /**< Private user data */
    voc_evt_data_t evt_data;            /**< Event specific data */
  };

  typedef struct voc_evt_filter
  {
    voc_lid_t lid;              /**< if a valid lid register for events related to this line only */
    voc_cid_t cid;              /**< if a valid cid register for events related to this call only */
  } voc_evt_filter_t;

/*****************************************

  Event Operations

 ******************************************/
/**
 *initialize VOC module.
 */
  voc_err_t voc_evt_initialize ();


/**
 *@brief Register a session-wide event type.
 *@param sid session ID
 *@param evt event type (VOC_EVT_LINE)
 *@param filter filter type
 *@param cb event callback function
 *@param user_data the user defined data will be passed back in the callback function
 *@return register operation result, 0 if successfull; otherwise a negative error code
 */
  voc_err_t voc_evt_register (voc_sid_t sid, voc_evt_type_t evt,
                              voc_evt_filter_t *filter, voc_cb_t cb,
                              void *user_data);

/**
 *@brief	Register for an event on a line.
 *@param sid session ID
 *@param lid line ID
 *@param evt event type (VOC_EVT_LINE_STATUS or VOC_EVT_LINE_CALL)
 *@param cb event callback function
 *@param user_data the user defined data witch will passed back in the callback function
 *@return 0 if successfull; otherwise a negative error code
 */
  voc_err_t voc_line_evt_register (voc_sid_t sid, voc_lid_t lid,
                                   voc_evt_type_t evt,
                                   voc_evt_filter_t *filter, voc_cb_t cb,
                                   void *user_data);


/**
 *@brief	Register for an event for a call.
 *@param sid session ID
 *@param cid call ID
 *@param evt event type (VOC_EVT_CALL_STATUS)
 *@param cb event callback function
 *@param user_data the user defined data witch will passed back in the callback function
 *@return 0 if successfull; otherwise a negative error code
 */
  voc_err_t voc_call_evt_register (voc_sid_t sid, voc_cid_t cid,
                                   voc_evt_type_t evt,
                                   voc_evt_filter_t *filter, voc_cb_t cb,
                                   void *user_data);


/**
 *@brief	Cease registration for a specific session-wise event.
 *@param sid session ID
 *@param evt event type
 *@return 0 if successfull; otherwise a negative error code
 */
  voc_err_t voc_evt_unregister (voc_sid_t sid, voc_evt_type_t evt);

/**
 *@brief	Cease registration for an event on a prticular line.
 *@param sid session ID
 *@param lid line id
 *@param evt event type
 *@return 0 if successfull; otherwise a negative error code
 */
  voc_err_t voc_line_evt_unregister (voc_sid_t sid, voc_lid_t lid,
                                     voc_evt_type_t evt);

/**
 *@brief	Cease registration for an event for a particular call.
 *@param sid session ID
 *@param cid call id
 *@param evt event type
 *@return 0 if successfull; otherwise a negative error code
 */
  voc_err_t voc_call_evt_unregister (voc_sid_t sid, voc_cid_t cid,
                                     voc_evt_type_t evt);



/**
 *@brief Delivers an event to the client by calling the callback function specified
 *through the corresponding event_register request. The callback function is called in the context of the event_process request.
 *@param tel_sid_t session ID
 *@param timeout the timeout of the event pending
 *@return the result of the operation
 */

  voc_err_t voc_evt_process (voc_sid_t sid, int timeout);


/**
 *@brief Package the event from DBUS
 *@param sid session ID
 *@param evt the event to pend
 *@param timeout the timeout of the event pending
 *@return the result of the operation
 */
  voc_err_t voc_evt_get (voc_sid_t sid, voc_evt_t **evt, int timeout);


/**
 *@brief Invokes the callback function contained in the descriptor.
 *@param sid session ID
 *@param evt the event in the operation
 *@return the result of the operation
 */
  voc_err_t voc_evt_call (voc_sid_t sid, voc_evt_t *evt);

/**
 *@brief Release all resources allocated for the event.
 *@param sid session ID
 *@param evt the event in the operation
 *@return the result of the operation
 */
  voc_err_t voc_evt_release (voc_sid_t sid, voc_evt_t *evt);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_VOC_EVENT_H */

/* 
vi:ts=2:nowrap:ai:expandtab
*/

