/* 	libvocenabler: - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	         YU Yijun <yijun.yu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef	__LIPS_VOC_SESSION_MUL_H
#define	__LIPS_VOC_SESSION_MUL_H
#include "voctype.h"
#include "vocerror.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  typedef enum
  {
    VOC_GSM = 0,

    VOC_WIFI,

    VOC_INVALID_ID,

  } voc_linetype_t;

#define SESSION_RANGE		100


/**
 *@brief Call status
 */
  typedef enum
  {
    VOC_CALL_IDLE,              /**< newly created or after been rejected or hung up */

    VOC_CALL_OUTGOING,     /**call is being routed to the remote peer */

    VOC_CALL_OUTGOING_DELIVERD,         /**< the call has reached destination */

    VOC_CALL_INCOMING,          /**< it's an incoming call delivered from the network */

    VOC_CALL_ACTIVE,                    /**< the call is established and active */

    VOC_CALL_HELD,      /**<call is established but held */


  } voc_call_state_t;

  typedef enum
  {
    VOC_LINE_REGISTERED,
    VOC_LINE_UNREGISTERED,
  } voc_line_reg_status_t;

/* ==================================================================================================
  FUNCTION PROTOTYPES
  ==================================================================================================*/
/**
 *initialize VOC module.
 */
  voc_err_t voc_initialize ();

/**
 *@brief Open a session with the voice call enabler
 *
 *@param sidp	a pointer to the location where in the case of success,
 *		a Session ID will be returned 
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_open_session (voc_lid_t index, voc_sid_t *sid);


/**
 *@brief Close a session with the voice call enabler
 *
 *@param sid 	the target Session ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_close_session (voc_sid_t sid);

/**
 *@brief Connect to a line
 *@param sid 	the Session ID
 *@param i	the target line index. All lines are indexed from 0 to n.
 *@param lidp	a pointer to the location where, in the case of success, 
 *		the target Line ID will be returned.
 *@return 0 if successfull; 
 *	VOC_ERR_NOT_EXIST
 *		no line with the specified index exist 
 *		but lines with higher indexes may still exist
 *	VOC_ERR_TOO_BIG
 *		no lines exist with any index equal to or higher than the specified @param i
 *@Note if a line is already connected and later connected again by voc_line_connect(), 
 *   then this function does nothing and successfully returns the target line ID again.
 */
  voc_err_t voc_line_connect (voc_sid_t sid, uint32_t i, voc_lid_t *lidp);

/**
 *@brief Disconnect from a line
 **@param sid 	the Session ID
 *@param lid	the target Line ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_line_disconnect (voc_sid_t sid, voc_lid_t lid);

 /**really register a line to the network. Before calling this func, app
 should call voc_line_is_registered first to check whether or not this line
 is already registered.
 
voc_err_t voc_line_register (voc_sid_t sid, voc_lid_t lid);
*/


/**
 *@brief get a specified line by name. LiPS standardizes some Line names,
 *such as GSM, VOIP. And an app can get a line directly by the well-known line name
 *@param sid the Session ID
 *@param name line name
 *@param lidp returned value, pointer to the target Line ID
 */
  voc_err_t voc_line_connect_by_name (voc_sid_t sid, const char *name,
                                      voc_lid_t *lidp);

/**
 *@brief Return a printable name of a line.
 *@param sid 	the Session ID
 *@param lid	the target Line ID
 *@param namep	a pointer to the location where, in the case of success, 
 *		a pointer to the string containing the name of the line will be returned.
 *@return 0 if successfull; otherwise a negative error code 
 *@Note Note that it's the responsability of the caller to free the string's memory eventually.
 */

  voc_err_t voc_line_get_name (voc_sid_t sid, voc_lid_t lid, char **namep);

/**
 *@brief Return a short printable secription of a line.
 *@param sid 	the Session ID
 *@param lid	the target Line ID
 *@param descp	a pointer to the location where, in the case of success, 
 *		a pointer to the string containing the description of the line will be returned.
 *@return 0 if successfull; otherwise a negative error code 
 *@Note		Note that it's the responsability of the caller to free the string's memory eventually.
 *
 */
  voc_err_t voc_line_get_description (voc_sid_t sid, voc_lid_t lid,
                                      char **descp);

/**
 *@brief Return the line's local end identification (e.g. phone number or URI).
 *@param sid 	the Session ID
 *@param lid	the target Line ID
 *@param local a pointer to the location where, in the case of success, 
 *		a pointer to the string containing the line's local identification will be returned.
 *@return 0 if successfull; otherwise a negative error code 
 *@Note Note that it's the responsability of the caller to free the string's memory eventually.
 */
  voc_err_t voc_line_get_local_address (voc_sid_t sid, voc_lid_t lid,
                                        char **local);

/**
 *@brief Return the status of a line
 *@param sid the Session ID
 *@param lid the target Line ID
 *@param signal a pointer to the location where, in the case of success, a pointer to the current
 *  status of the line will be returned.
 *@return 0 if successful; otherwise a negative error code
 */
  voc_err_t voc_line_get_signal (voc_sid_t sid, voc_lid_t lid,
                                 voc_line_signal_t *signal);

/**
 *@brief	create and initiate a call to a remote peer.
 *@param sid 	the Session ID
 *@param lid	the target line id. a call will be created from this line.
 *@param cid	the ID of newly created outgoing Call
 *@param remote designate the remote peer. Could be a number of an URI.
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_dial (voc_sid_t sid, voc_lid_t lid, voc_cid_t *cid,
                      const char *remote);

/**
 *@brief Triggers transition of an active call to onhold state.
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_onhold (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief Triggers transition of a onhold call to active state.
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_unhold (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief Triggers establishment of a connection between peers.
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_accept (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief Triggers the switch between mute and unmute
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_mute (voc_sid_t sid, UInt32 is_mute);


/**
 *@brief Triggers the operation of hungup the call 
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_hungup (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief Triggers the operation of hungup all calls
 *
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_hungup_all (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief Triggers cancelation of an outgoing call.
 *
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_hungup_outgoing (voc_sid_t sid, voc_cid_t cid);

/**brief Triggers cancelation of an active call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_hungup_active (voc_sid_t sid, voc_cid_t cid);

/**Triggers termination of the call on hold.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_hungup_held (voc_sid_t sid, voc_cid_t cid);


/**Triggers operation of accepting an incoming call, at the same time, terminating the hold call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_replace_hold (voc_sid_t sid, voc_cid_t cid);

/**Triggers operation of put all calls in conference.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_conference (voc_sid_t sid, voc_cid_t cid);

/**Triggers operation of make all calls exit conference.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_exit_conference (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_send_dtmf (voc_sid_t sid, voc_cid_t cid, char *number);

/**
 *@brief	get the start time of a call
 *@param sid the Session ID
 *@param cid	the target incoming Call ID
 *@param start pointer to the time structure which contains the start time
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_get_start_time (voc_sid_t sid, voc_cid_t cid,
                                voc_time_t *start);

/**
 *@brief	get the end time of a call
 *@param sid the Session ID
 *@param cid	the target incoming Call ID
 *@param end pointer to the time structure which contains the end time
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_get_end_time (voc_sid_t sid, voc_cid_t cid, voc_time_t *end);

/**
 *@brief	Return the remote number of a call.
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *@param remote	returned remote phone number.
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_get_remote (voc_sid_t sid, voc_cid_t cid, char **remote);


#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_VOC_SESSION_MUL_H */

/* 
vi:ts=2:nowrap:ai:expandtab
*/

