/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include "dbus-if.h"
#include "shared.h"
#include "vocerror.h"
#include "voctype.h"


static GList *listeners = NULL;
static DBusConnection *bus = NULL;

typedef struct
{
  gchar *interface;
  gchar *method;
  void (*callback) ();
} t_listener;

static DBusHandlerResult
msg_filter (DBusConnection *connection, DBusMessage *message, void *user_data)
{

  /* User data is the event loop we are running in */
  GMainLoop *loop = (GMainLoop *) user_data;
  GList *iter;

  for (iter = listeners; iter; iter = iter->next)
  {
    t_listener *li = (t_listener *) iter->data;

    if (dbus_message_is_signal (message, li->interface, li->method))
    {
      li->callback (message);
      return DBUS_HANDLER_RESULT_HANDLED;
    }
    else
    {
      if (dbus_message_is_method_call (message, li->interface, li->method))
      {
        li->callback (message);
        return DBUS_HANDLER_RESULT_HANDLED;
      }
    }

  }

  /* A signal from the bus saying we are about to be disconnected */
  if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected"))
  {
    /* Tell the main loop to quit */
    g_main_loop_quit (loop);

    /* We have handled this message, don't pass it on */
    return DBUS_HANDLER_RESULT_HANDLED;
  }

  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

void
dbus_add_msg_listener (const gchar *interface, const gchar *method,
                       void (*callback) ())
{

  t_listener *new_listener = g_malloc0 (sizeof (t_listener));
  gchar *filterstr =
    g_strdup_printf ("type='signal',interface='%s'", interface);

  new_listener->interface = g_strdup (interface);
  new_listener->method = g_strdup (method);
  new_listener->callback = callback;

  listeners = g_list_append (listeners, (gpointer) new_listener);

  dbus_bus_add_match (bus, filterstr, NULL);
  g_free (filterstr);
}


void
dbus_add_msg_listener_mc (const gchar *interface, const gchar *method,
                          void (*callback) ())
{
//fix me, using hash table may be better.

  t_listener *new_listener = g_malloc0 (sizeof (t_listener));
  gchar *filterstr =
    g_strdup_printf ("type='method_call',interface='%s'", interface);

  new_listener->interface = g_strdup (interface);
  new_listener->method = g_strdup (method);
  new_listener->callback = callback;

  listeners = g_list_append (listeners, (gpointer) new_listener);

  dbus_bus_add_match (bus, filterstr, NULL);

  g_free (filterstr);

}


voc_err_t
voc_dbus_connect (UInt32 *sid)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;

  dbus_error_init (&error);

  g_print ("in %s,before msg\n", __FUNCTION__);

  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_OBJECT,
                                          PHONE_SERVER_INTERFACE,
                                          PHONE_SERVER_DBUS_CONNECT);
  if (!message)
  {
    g_print ("Failed to contruct message call");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }
  
  /*
  TO do
  1. Should also append args including: Unique name, interface, method
  2. Then PS will add the items to the table it maintains, in which key=sid, value= Unique, interface, method
  3. PS send message to TAPI
  4. PS get message from TAPI
  5. PS return sid back
  dbus_error_init (&error);
  */

  reply =
    dbus_connection_send_with_reply_and_block (bus, message, 10000, &error);
  if (dbus_error_is_set (&error))
  {
    g_print
      ("!!!FAIL:EVENT Message received, but error getting message: %s\n",
       error.message);
    g_print ("Failed to connect to phone server\n");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }
  g_print ("in %s,after msg send, before msg get\n", __FUNCTION__);
  if (dbus_message_get_args (reply, &error, DBUS_TYPE_UINT32, sid,
                             //DBUS_TYPE_STRING, &sr,
                             DBUS_TYPE_INVALID))
  {
    g_print ("connect success with sid= %d!\n", *sid);
  }
  else
  {
    g_print ("message received with error: %s\n", error.message);

    g_print ("Failed to connect to phone server\n");
    dbus_error_free (&error);
    dbus_message_unref (message);
    dbus_message_unref (reply);
    return VOC_ERR_CARRIER;
  }
  dbus_message_unref (message);
  dbus_message_unref (reply);

  g_print ("Connected to phone server successfully.\n");

  //dbus_add_msg_listener_mc ( PHONE_VOICE_INTERFACE, VOC_DBUS_METHOD_SEND_EVENT, dbus_cb);


  return VOC_ERR_NONE;

}

voc_err_t
voc_dbus_disconnect (UInt32 sid)
{
//TO do
// 1. notify PS to delete the item in the table
  if (!bus)
    return VOC_ERR_NONE;

  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  DBusError error;

  dbus_error_init (&error);
  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_OBJECT,
                                          PHONE_SERVER_INTERFACE,
                                          PHONE_SERVER_DBUS_DISCONNECT);

  if (!message)
  {
    g_print ("Failed to contruct message call with error %s:\n", error.message);
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }

  dbus_message_append_args (message, DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  reply =
    dbus_connection_send_with_reply_and_block (bus, message, 2000, &error);
  //dbus_message_unref(message);

  if (dbus_error_is_set (&error))
  {
    g_print ("Failed to disconnect from phone server\n");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }
  int sid_get;

  if (dbus_message_get_args (reply, &error,
                             DBUS_TYPE_UINT32, &sid_get, DBUS_TYPE_INVALID))
  {
    g_print ("Disconnect success with sid=%d!\n", sid_get);
  }
  else
  {
    g_print ("message received with error: %s\n", error.message);
    g_print ("Failed to connect to phone server\n");

    dbus_error_free (&error);
    dbus_message_unref (message);
    dbus_message_unref (reply);
    return VOC_ERR_CARRIER;
  }
  dbus_message_unref (message);
  dbus_message_unref (reply);

  dbus_connection_close (bus);
  dbus_connection_unref (bus);
  g_print ("Disconnected from phone server successfully\n");

  return VOC_ERR_NONE;

}




DBusConnection *
dbus_init ()
{
  DBusError error;
  GMainLoop *mainloop;

  mainloop = g_main_loop_new (NULL, TRUE);
  dbus_error_init (&error);

  bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
  if (bus == NULL)
  {
    g_printerr ("Failed to get session bus connection %s", error.message);
    dbus_error_free (&error);
    return NULL;
  }

  dbus_connection_setup_with_g_main (bus, NULL);

  /* we do not request a namespace to support multiple listeners to these signals */
  /* listening to messages from all objects as no path is specified */


  dbus_connection_add_filter (bus, msg_filter, mainloop, NULL);
//dbus_add_msg_listener_mc ( PHONE_VOICE_INTERFACE, VOC_DBUS_METHOD_SEND_EVENT, cb_dial2);


  //g_main_loop_unref(mainloop);

  return bus;
}


/* 
vi:ts=2:nowrap:ai:expandtab
*/

