/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <glib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include "dbus-if.h"
#include "shared.h"

#include "tapi-gsm.h"
#include "vocevent.h"

static DBusConnection *session_bus = NULL;

tel_gsm_err_t tel_gsm_evt_get   (voc_sid_t sid, tel_gsm_evt_t **evt, Int32 timeout);
tel_gsm_err_t tel_gsm_evt_call  (voc_sid_t sid, tel_gsm_evt_t *evt);

static void
cb_psevt (DBusMessage *message)
{
  g_print ("!!!!!!!!!888888event from Phoneserver received!\n");

  DBusError error;
  DBusMessage *reply;

  dbus_error_init (&error);
  tel_gsm_evt_type_t event_dbus;
  voc_sid_t sid_dbus;
  UInt32 lcid_dbus;
  Int8 *peer_number;
  tel_gsm_evt_t *evt;

  evt = g_new0 (tel_gsm_evt_t, 1);


  if (!dbus_message_get_no_reply (message))     //sender expects a reply
  {
    reply = dbus_message_new_method_return (message);
    dbus_connection_send (session_bus, reply, NULL);
    dbus_message_unref (reply);
  }

  if (dbus_message_get_args (message, &error,
                             DBUS_TYPE_UINT32, &event_dbus,
                             DBUS_TYPE_UINT32, &sid_dbus,
                             DBUS_TYPE_UINT32, &lcid_dbus,
                             DBUS_TYPE_STRING, &peer_number, DBUS_TYPE_INVALID))
  {
    g_print ("event type=%d, ", event_dbus);
    g_print ("sid=%d, ", sid_dbus);
    g_print ("lcid=%d, ", lcid_dbus);
    g_print ("peer_number=%s\n", peer_number);



    evt->evt_type = event_dbus;

    //evt->user_data=(void *)peer_number;

    evt->evt_data = lcid_dbus;

    if (event_dbus == TEL_EVT_LINE_CALL)
    {
      evt->user_data = (void *) peer_number;
      //change the evt_data from cid to lid
      voc_lid_t lid_call;
      voc_cid_t cid_dbus;

      cid_dbus = evt->evt_data;
      voc_call_get_line (sid_dbus, cid_dbus, &lid_call);
      evt->user_data = peer_number;
      if (!tel_gsm_evt_get (sid_dbus, &evt, 0))
      {
        g_print ("evt->type=%d", evt->evt_type);
        g_print ("evt->data=%d\n", evt->evt_data);
        g_print ("user->data=%s\n", (char *) evt->user_data);


        tel_gsm_evt_call (sid_dbus, evt);
        //voc_call_evt_unregister(sid_dbus, lcid_dbus,evt->evt_type);
        /* sxh: here no need to release */
        //tel_gsm_evt_release (sid_dbus, evt);
        g_print ("###LINE CALL ONCE#################\n");
      }
    }

    else
    {
      printf ("it is a call _status event!!!!!!!!!!!!!!!!!!!!!!!\n");
      if (!tel_gsm_evt_get (sid_dbus, &evt, 0))
      {
#ifdef DEBUG
        g_print ("evt->type=%d", evt->evt_type);
        g_print ("evt->data=%d\n", evt->evt_data);
#endif

        tel_gsm_evt_call (sid_dbus, evt);

        //voc_call_evt_unregister(sid_dbus, lcid_dbus,evt->evt_type);
        /* sxh: here no need to release */
        //tel_gsm_evt_release (sid_dbus, evt);
        g_print ("####################\n");
      }
    }
  }
  else
  {
    g_print ("EVENT Message received, but error getting message: %s\n",
             error.message);
    dbus_error_free (&error);
  }
}



static void
cb_unspsevt (DBusMessage *message)
{
  g_print ("!!!!!!!!!9999999 UNSOLICATED event from Phoneserver received!\n");

  DBusError error;
  DBusMessage *reply;
  voc_sid_t sid_dbus;
  voc_cid_t cid_dbus;


  dbus_error_init (&error);

  if (!dbus_message_get_no_reply (message))
  {                             //sender expects a reply             
    reply = dbus_message_new_method_return (message);
    dbus_connection_send (session_bus, reply, NULL);
    dbus_message_unref (reply);
  }

  if (dbus_message_get_args (message, &error,
                             DBUS_TYPE_UINT32, &sid_dbus,
                             DBUS_TYPE_UINT32, &cid_dbus, DBUS_TYPE_INVALID))
  {
    g_print ("sid=%d, ", sid_dbus);
    g_print ("cid=%d, ", cid_dbus);

    //voc_call_evt_unregister(0, cid_dbus, 4);
    tel_gsm_evt_t *evt;

    evt = g_new0 (tel_gsm_evt_t, 1);
    evt->evt_data = cid_dbus;
    evt->evt_type = TEL_EVT_CALL_REMOTE_STATUS;


    if (!tel_gsm_evt_get (sid_dbus, &evt, 0))
    {
      g_print ("evt->type=%d", evt->evt_type);
      g_print ("evt->data=%d\n", evt->evt_data);

      tel_gsm_evt_call (sid_dbus, evt);

      //voc_evt_release (sid_dbus, evt);
      g_print ("####################\n");
    }

  }
  else
  {
    g_print ("EVENT Message received, but error getting message: %s\n",
             error.message);
    dbus_error_free (&error);
  }
}

static gboolean
send_evt_to_dbus_mc_dial (DBusConnection *bus,
                          voc_evt_type_t evt,
                          voc_sid_t sid,
                          voc_lid_t lid,
                          voc_cid_t *cid_create, const char *remote)
    //char*pmethod)
{
  DBusMessage *message;
  DBusError error;

  dbus_error_init (&error);



  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_VOICE_OBJECT,
                                          PHONE_SERVER_VOICE_INTERFACE, "dial");

  if (message == NULL)
  {
    printf ("Failed to contruct message\n");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }

  /* Append the event to the signal 
  when method=dial, the lid not cid is added
  in other ocasions, cid is added
  */
  dbus_message_append_args (message, DBUS_TYPE_UINT32, &evt,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_UINT32, &lid, DBUS_TYPE_STRING, &remote,
                            //DBUS_TYPE_STRING, &pmethod,
                            DBUS_TYPE_INVALID);

  DBusMessage *reply = NULL;

  dbus_error_init (&error);
  reply =
    dbus_connection_send_with_reply_and_block (bus, message, 2000, &error);
  if (dbus_error_is_set (&error))
  {
    g_print ("Could not get create a new call!error: %s\n", error.message);
    dbus_error_free (&error);

    return VOC_ERR_CARRIER;
  }
  if (dbus_message_get_args (reply, &error,
                             DBUS_TYPE_UINT32, cid_create, DBUS_TYPE_INVALID))
  {
    g_print ("successfully getting cid = %d!\n", *cid_create);
  }
  else
  {
    g_print ("error getting cid with error: %s\n", error.message);
    dbus_error_free (&error);
    dbus_message_unref (message);
    dbus_message_unref (reply);
    return VOC_ERR_CARRIER;
  }
  dbus_message_unref (message);
  dbus_message_unref (reply);

  g_print ("Message sendout!\n");
  /* Return TRUE to tell the event loop we want to be called again */
  return VOC_ERR_NONE;
}



static gboolean
send_evt_to_dbus_mc_hold_dial (DBusConnection *bus,
                               voc_evt_type_t evt,
                               voc_sid_t sid,
                               voc_lid_t lid,
                               voc_cid_t *cid_create, const char *remote)
{
  DBusMessage *message;
  DBusError error;

  dbus_error_init (&error);



  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_VOICE_OBJECT,
                                          PHONE_SERVER_VOICE_INTERFACE,
                                          "hold_dial");

  if (message == NULL)
  {
    printf ("Failed to contruct message\n");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }

  dbus_message_append_args (message, DBUS_TYPE_UINT32, &evt,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_UINT32, &lid, DBUS_TYPE_STRING, &remote,
                            //DBUS_TYPE_STRING, &pmethod,
                            DBUS_TYPE_INVALID);

  DBusMessage *reply = NULL;

  dbus_error_init (&error);
  reply =
    dbus_connection_send_with_reply_and_block (bus, message, 2000, &error);
  if (dbus_error_is_set (&error))
  {
    g_print ("Could not get create a new call!error: %s\n", error.message);
    dbus_error_free (&error);

    return VOC_ERR_CARRIER;
  }
  if (dbus_message_get_args (reply, &error,
                             DBUS_TYPE_UINT32, cid_create, DBUS_TYPE_INVALID))
  {
    g_print ("successfully getting cid = %d!\n", *cid_create);
  }
  else
  {
    g_print ("error getting cid with error: %s\n", error.message);
    dbus_error_free (&error);
    dbus_message_unref (message);
    dbus_message_unref (reply);
    return VOC_ERR_CARRIER;
  }
  dbus_message_unref (message);
  dbus_message_unref (reply);

  g_print ("Message sendout!\n");
  return VOC_ERR_NONE;
}


static gboolean
send_evt_to_dbus_mc (DBusConnection *bus,
                     voc_evt_type_t evt,
                     voc_sid_t sid,
                     voc_cid_t cid, const char *remote, char *pmethod)
{
  DBusMessage *message;
  DBusError error;

  dbus_error_init (&error);

  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_VOICE_OBJECT,
                                          PHONE_SERVER_VOICE_INTERFACE,
                                          pmethod);

  if (message == NULL)
  {
    printf ("Failed to contruct message\n");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }


  /* Append the event to the signal 
  when method=dial, the lid not cid is added
  in other ocasions, cid is added  */
  dbus_message_append_args (message, DBUS_TYPE_UINT32, &evt,
                            DBUS_TYPE_UINT32, &sid, DBUS_TYPE_UINT32, &cid,
                            //DBUS_TYPE_STRING, &remote,    
                            DBUS_TYPE_STRING, &pmethod, DBUS_TYPE_INVALID);


  dbus_message_set_no_reply (message, TRUE);
  dbus_connection_send (bus, message, NULL);

  dbus_message_unref (message);


  g_print ("%s Message sendout!\n", pmethod);
  /* Return TRUE to tell the event loop we want to be called again */
  return VOC_ERR_NONE;
}





static gboolean
send_evt_to_dbus_mc_dtmf (DBusConnection *bus,
                          voc_evt_type_t evt,
                          voc_sid_t sid, voc_cid_t cid, char *dtmf)
{
  DBusMessage *message;
  DBusError error;

  dbus_error_init (&error);

  message = dbus_message_new_method_call (PHONE_SERVER_SERVICE,
                                          PHONE_SERVER_VOICE_OBJECT,
                                          PHONE_SERVER_VOICE_INTERFACE, "dtmf");

  if (message == NULL)
  {
    printf ("Failed to contruct message\n");
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }


  /* Append the event to the signal 
  when method=dial, the lid not cid is added
  in other ocasions, cid is added*/
  dbus_message_append_args (message, DBUS_TYPE_UINT32, &evt,
                            DBUS_TYPE_UINT32, &sid, DBUS_TYPE_UINT32, &cid,
                            DBUS_TYPE_STRING, &dtmf, DBUS_TYPE_INVALID);


  dbus_message_set_no_reply (message, TRUE);
  /* Send the signal */
  dbus_connection_send (bus, message, NULL);

  /* Free the signal now we have finished with it */
  dbus_message_unref (message);


  g_print ("Message sendout!\n");
  /* Return TRUE to tell the event loop we want to be called again */
  return VOC_ERR_NONE;
}

/* 
 *	a function to create a hash value from a key.
 */
guint
tel_gsm_evt_table_hash (gconstpointer key)
{
  guint value = *(guint *) key;

  return value;
}

/* 
 *	a function to check two keys for equality.
 */
gboolean
tel_gsm_evt_table_equal (gconstpointer a, gconstpointer b)
{
  return *(guint *) a == *(guint *) b;
}

/* 
 *	a function to free the memory allocated for the 
 *  value used when removing the entry from the GHashTable.
 */
void
tel_gsm_evt_table_destroy_value (gpointer data)
{
  g_list_free ((GList *) data);
}


tel_gsm_err_t
tapi_gsm_initialize ()
{
  DBusError error;

  tel_register_evt_table =
    g_hash_table_new_full (tel_gsm_evt_table_hash, tel_gsm_evt_table_equal,
                           NULL, tel_gsm_evt_table_destroy_value);
  session_bus = dbus_init ();
  if (!session_bus)
  {
    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
    dbus_error_free (&error);
    return VOC_ERR_CARRIER;
  }
  return VOC_ERR_NONE;
}


tel_gsm_err_t
tapi_gsm_finalize ()
{
  g_hash_table_destroy (tel_register_evt_table);
  return VOC_ERR_CARRIER;
}

Boolean
tel_gsm_if_registered (voc_sid_t sid, tel_gsm_evt_type_t evt_type)
{
  g_print ("in %s, evtype= %d, \n", __FUNCTION__, evt_type);

  GList *list = NULL;
  GList *nextem = NULL;
  tel_gsm_register_evt_t *event;

  /* search sid in the event register list */
  list = (GList *) g_hash_table_lookup (tel_register_evt_table, &evt_type);
  g_print ("in gsm evt_get, before if, evt_type=%d \n", evt_type);
  g_print ("list=[0x%08x]\n", (guint32) list);


  if (list != NULL)
  {
    nextem = list->next;
    //nextem=list;
    while (nextem)
    {
      event = (tel_gsm_register_evt_t *) nextem->data;
      //if(event->sid == sid)
      if (event->sid == sid)
      {
        /* store the event */
        tel_event_registered = event;
        return TRUE;
      }
      nextem = nextem->next;
    }
  }
  return FALSE;
}


tel_gsm_err_t
tel_gsm_evt_register (voc_sid_t sid,
                      tel_gsm_evt_type_t evt_type,
                      tel_gsm_cb_t cb, void *user_data)
{
  GList *list = NULL;

  tel_gsm_evt_type_t *key = NULL;

  key = g_new0 (tel_gsm_evt_type_t, 1);
  *key = evt_type;

  tel_gsm_register_evt_t *event;

  event = g_new0 (tel_gsm_register_evt_t, 1);
  event->sid = sid;
  event->cb = cb;
  event->user_data = user_data;
  /* If we have already registered that event, simply return */
  if (tel_gsm_if_registered (sid, evt_type) == TRUE)
  {
    return TEL_ERR_NONE;
  }
  else
  {
    /* first check if the hash key for event type is used,
       if used, add the event in the GPtrArray of that key,
       otherwise insert a new key-value pair for that event type */
    list = (GList *) g_hash_table_lookup (tel_register_evt_table, key);
    if (list == NULL)
    {
      list = g_list_alloc ();
      list = g_list_append (list, event);
      g_hash_table_insert (tel_register_evt_table, key, list);
      g_print
        ("in tel_gsm_evt_register, after register, evt_type=%d, cb=[0x%08x] \n",
         evt_type, (guint32) cb);

    }
    else
    {
      g_list_append (list, event);
    }
  }
  return TEL_ERR_NONE;
}




/**
 *Deregister an unsolicited event type.
 *@param sid session ID
 *@param evt unsolicited event type
 *@return deregister operation result
 */
tel_gsm_err_t
tel_gsm_evt_unregister (voc_sid_t sid, tel_gsm_evt_type_t evt_type)
{
  GList *list = NULL;
  GList *next = NULL;
  tel_gsm_register_evt_t *event;

  list = (GList *) g_hash_table_lookup (tel_register_evt_table, &evt_type);
  /* now we remove the event with sid from the list */
  if (list != NULL)
  {
    //next = list;
    next = list->next;          //1205
    while (next)
    {
      event = (tel_gsm_register_evt_t *) next->data;
      if (event->sid == sid)
      {
        g_list_remove (list, event);
        g_free (event->user_data);
        g_free (event);

        return TEL_ERR_NONE;
      }
      next = next->next;
    }
  }
  return TEL_ERR_GENERAL;
}


tel_gsm_err_t
tel_gsm_evt_get (voc_sid_t sid, tel_gsm_evt_t **evt, Int32 timeout)
{

  g_print ("in %s, evtype= %d, \n", __FUNCTION__, (*evt)->evt_type);
  if (!tel_gsm_if_registered (sid, (*evt)->evt_type))
  {
    return TEL_ERR_NOTREGISTERED;
  }

  /* 
   *     Process message
   */
  (*evt)->cb = tel_event_registered->cb;
  if ((*evt)->evt_type != TEL_EVT_LINE_CALL)
    (*evt)->user_data = tel_event_registered->user_data;


  printf ("after evt get, cb=0x%08x\n", (*evt)->cb);


  return TEL_ERR_NONE;
}


tel_gsm_err_t
tel_gsm_evt_call (voc_sid_t sid, tel_gsm_evt_t *evt)
{
  tel_gsm_cb_t cb;

  cb = (tel_gsm_cb_t) evt->cb;
  g_print (" |event_call    : 0x%08x\n", (guint32) cb);
  (*cb) (sid, evt);
  return TEL_ERR_NONE;
}


tel_gsm_err_t
tel_gsm_evt_release (voc_sid_t sid, tel_gsm_evt_type_t *evt)
{
  if (!evt)
    return TEL_ERR_NONE;
  g_free (evt);
  return TEL_ERR_NONE;
}


/**Open a session.
 *
 *@param sid	a pointer to the location where in the case of success,
 *		a Session ID will be returned 
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_gsm_err_t
tel_gsm_open_session (UInt32 *sid)
{

  voc_dbus_connect (sid);

  dbus_add_msg_listener_mc (PHONE_VOICE_INTERFACE,
                            PHONE_VOICE_DELIVER_EVENT, cb_psevt);
  dbus_add_msg_listener_mc (PHONE_VOICE_INTERFACE,
                            PHONE_VOICE_DELIVER_UNSEVENT, cb_unspsevt);
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_close_session (voc_sid_t sid)
{
  if (voc_dbus_disconnect (sid) == VOC_ERR_NONE)
    return TEL_ERR_NONE;
  else
    return TEL_ERR_DBUS;
}


tel_gsm_err_t
tel_gsm_call_exit_conference (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;
  char *pmethod = "exit_conference";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_conference (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;
  char *pmethod = "conference";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  //voc_call_evt_register (sid, cid, evt, NULL, voc_cb_conference, NULL);//cb1 is used to change the UI, etc.
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_hungup_held (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  //voc_call_evt_register (sid, cid, evt, NULL, voc_cb_hungup_held, NULL);//cb1 is used to change the UI, etc.
  char *pmethod = "hungup_held";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_hungup_outgoing (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  char *pmethod = "hungup_outgoing";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_replace_hold (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  //voc_call_evt_register (sid, cid, evt, NULL, voc_cb_replace_hold, NULL);//cb1 is used to change the UI, etc.
  char *pmethod = "replace_held";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_hungup_active (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  //voc_call_evt_register (sid, cid, evt, NULL, voc_cb_hungup_active, NULL);//cb1 is used to change the UI, etc.
  char *pmethod = "hungup_active";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_hungup_all (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;
  char *pmethod = "hungup_all";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_hungup (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  char *pmethod = "hungup";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_accept (voc_sid_t sid, voc_cid_t cid)
{
  g_print ("Enter the tel_gsm_call_accept,cid=%d\n\n\n", cid);
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  //voc_call_evt_register (sid, cid, evt, NULL, voc_cb_accept, NULL);//cb1 is used to change the UI, etc.
  char *pmethod = "accept";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}


tel_gsm_err_t
tel_gsm_call_onhold (voc_sid_t sid, voc_cid_t cid)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;
  char *pmethod = "onhold";

  if (send_evt_to_dbus_mc (session_bus, evt, sid, cid, NULL, pmethod))
    return TEL_ERR_DBUS;
  //voc_call_evt_register (sid, cid, evt, NULL, voc_cb_onhold, NULL);//cb1 is used to change the UI, etc.
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_mute (voc_sid_t sid, UInt32 is_mute)
{

  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;
  char *pmethod = "mute";

  g_print ("The mute paramaters equals to: %d\n", is_mute);

  if (send_evt_to_dbus_mc (session_bus, evt, sid, is_mute, NULL, pmethod))      //, pmethod))
    return TEL_ERR_DBUS;
  //voc_evt_register (sid,  evt, NULL, voc_cb_mute, NULL);//cb1 is used to change the UI, etc.
  return TEL_ERR_NONE;
}

tel_gsm_err_t
tel_gsm_call_send_dtmf (voc_sid_t sid, voc_cid_t cid, char *number)
{
  //voc_call_create (sid, lid, cid);

  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  if (number == NULL)
    return TEL_ERR_GENERAL;
  if (send_evt_to_dbus_mc_dtmf (session_bus, evt, sid, cid, number))    //, pmethod))
    return TEL_ERR_DBUS;
  // voc_call_evt_register (sid, *cid, evt, NULL, voc_cb_dial, (void *)remote);//cb1 is used to change the UI, etc.
  return TEL_ERR_NONE;

}


tel_gsm_err_t
tel_gsm_call_dial (voc_sid_t sid, voc_lid_t lid,
                   voc_cid_t *cid, const char *remote)
{
  tel_gsm_evt_type_t evt = TEL_EVT_CALL_STATUS;

  if (send_evt_to_dbus_mc_dial (session_bus, evt, sid, lid, cid, remote))       //, pmethod))
    return TEL_ERR_DBUS;
  return TEL_ERR_NONE;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

