/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <glib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include "shared.h"

#include "tapi-wifi.h"
#include "vocevent.h"

/* TODO:this should be global */
static guint g_sessionid = 0;

/* this hold the iterate timeout id */
static guint timeout_id = 0;

static LinphoneCore linphonec;
static char configfile_name[PATH_MAX];

static void
stub ()
{
}

LinphoneCoreVTable linphonec_vtable = {
show:stub,
inv_recv:stub,
bye_recv:stub,
notify_recv:stub,
new_unknown_subscriber:stub,
auth_info_requested:stub,
display_status:stub,
display_message:stub,
display_warning:stub,
display_url:stub,
display_question:stub,
text_received:stub
};



static GHashTable *wifi_register_evt_table;
static tel_wifi_register_evt_t *wifi_event_registered;


static int
lpc_cmd_call (LinphoneCore *lc, char *args)
{
  printf ("in ipc 1\n");

  if (lc->call != NULL)
  {
    printf ("Terminate current call first.\n");
  }
  else
  {
    printf ("in ipc 2, %s\n", args);

    if (-1 == linphone_core_invite (lc, args))
    {
      printf ("Error from linphone_core_invite.\n");
    }
    else
    {
      /* current_call=args; */
    }
  }
  return 1;
}



static int
lpc_cmd_answer (LinphoneCore *lc, char *args)
{
  if (-1 == linphone_core_accept_call (lc, NULL))
  {
    printf ("No incoming call.\n");
  }
  return 1;
}




static int
lpc_cmd_terminate (LinphoneCore *lc, char *args)
{
  if (-1 == linphone_core_terminate_call (lc, NULL))
  {
    printf ("No active call.\n");
  }
  return 1;
}


/* 
 *	a function to create a hash value from a key.
 */
static guint
tel_wifi_evt_table_hash (gconstpointer key)
{
  guint value = *(guint *) key;

  return value;
}

/* 
 *	a function to check two keys for equality.
 */
static gboolean
tel_wifi_evt_table_equal (gconstpointer a, gconstpointer b)
{
  return *(guint *) a == *(guint *) b;
}

/* 
 *	a function to free the memory allocated for the 
 *  value used when removing the entry from the GHashTable.
 */
static void
tel_wifi_evt_table_destroy_value (gpointer data)
{
  return g_list_free ((GList *) data);
}

Boolean
tel_wifi_if_registered (voc_sid_t sid, tel_wifi_evt_type_t evt_type)
{
  GList *list = NULL;
  GList *nextem = NULL;
  tel_wifi_register_evt_t *event;

  /* search sid in the event register list */
  list = (GList *) g_hash_table_lookup (wifi_register_evt_table, &evt_type);
  g_print ("in evt_get, before if, evt_type=%d \n", evt_type);

  if (list != NULL)
  {
    nextem = list->next;
    //nextem=list;
    while (nextem)
    {
      event = (tel_wifi_register_evt_t *) nextem->data;
      //if(event->sid == sid)
      if (event->sid == sid)
      {
        /* store the event */
        wifi_event_registered = event;
        return TRUE;
      }
      nextem = nextem->next;
    }
  }
  return FALSE;
}


tel_wifi_err_t
tel_wifi_evt_register (voc_sid_t sid,
                       tel_wifi_evt_type_t evt_type,
                       tel_wifi_cb_t cb, void *user_data)
{
  GList *list = NULL;

  tel_wifi_evt_type_t *key = NULL;

  key = g_new0 (tel_wifi_evt_type_t, 1);
  *key = evt_type;


  tel_wifi_register_evt_t *event;

  event = g_new0 (tel_wifi_register_evt_t, 1);
  event->sid = sid;
  event->cb = cb;
  event->user_data = user_data;
  /* If we have already registered that event, simply return */
  if (tel_wifi_if_registered (sid, evt_type) == TRUE)
  {
    return WIFI_ERR_NONE;
  }
  else
  {
    /* first check if the hash key for event type is used,
       if used, add the event in the GPtrArray of that key,
       otherwise insert a new key-value pair for that event type */
    list = (GList *) g_hash_table_lookup (wifi_register_evt_table, key);
    if (list == NULL)
    {
      list = g_list_alloc ();
      list = g_list_append (list, event);
      g_hash_table_insert (wifi_register_evt_table, key, list);
    }
    else
    {
      g_list_append (list, event);
    }
  }
  return WIFI_ERR_NONE;
}


/**
 *Deregister an unsolicited event type.
 *@param sid session ID
 *@param evt unsolicited event type
 *@return deregister operation result
 */
voc_err_t
tel_wifi_evt_unregister (voc_sid_t sid, tel_wifi_evt_type_t evt_type)
{
  GList *list = NULL;
  GList *next = NULL;
  tel_wifi_register_evt_t *event;

  list = (GList *) g_hash_table_lookup (wifi_register_evt_table, &evt_type);
  /* now we remove the event with sid from the list */
  if (list != NULL)
  {
    next = list->next;          //1205
    while (next)
    {
      event = (tel_wifi_register_evt_t *) next->data;
      if (event->sid == sid)
      {
        g_list_remove (list, event);
        g_free (event->user_data);
        g_free (event);
        return WIFI_ERR_NONE;
      }
      next = next->next;
    }
  }
  return WIFI_ERR_GENERAL;
}

voc_err_t
tel_wifi_evt_get (voc_sid_t sid, tel_wifi_evt_t **evt, Int32 timeout)
{

  if (!tel_wifi_if_registered (sid, (*evt)->evt_type))
  {
    return WIFI_ERR_NOTREGISTERED;
  }

  /* 
   *     Process message
   */
  g_print ("in evt_get, after if \n");
  (*evt)->cb = wifi_event_registered->cb;
/* jd
	if ((*evt)->evt_type != WIFI_EVT_LINE_CALL)
		(*evt)->user_data = wifi_event_registered->user_data;
jdd*/

#ifdef DEBUG
  //printf ("type=%d, lid=%d, cid=%d\n", (*evt)->evt_type,lid,cid);
#endif



  return WIFI_ERR_NONE;
}

voc_err_t
tel_wifi_evt_call (voc_sid_t sid, tel_wifi_evt_t *evt)
{
  printf ("\n\t\tEnter the evt_call:0!!!!!!!!!!!!!,sid=%d\n", sid);


  tel_wifi_cb_t cb;

  cb = (tel_wifi_cb_t) evt->cb;
  g_print (" |event_call    : 0x%08x\n", (guint32) cb);

  (*cb) (sid, evt);
  return WIFI_ERR_NONE;
}

tel_wifi_err_t
tel_wifi_evt_release (voc_sid_t sid, tel_wifi_evt_t *evt)
{
  if (!evt)
    return WIFI_ERR_NONE;
  g_free (evt);
  return WIFI_ERR_NONE;
}

/**Open a session.
 *
 *@param sid	a pointer to the location where in the case of success,
 *		a Session ID will be returned 
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_wifi_err_t
tel_wifi_open_session (UInt32 *sid)
{
  g_sessionid++;
  linphonec.sid = g_sessionid;
  *sid = g_sessionid;
  return WIFI_ERR_NONE;
}

tel_wifi_err_t
tel_wifi_close_session (voc_sid_t sid)
{
  return WIFI_ERR_NONE;
}





static gboolean
tapi_wifi_call_iterate ()
{
  linphone_core_iterate (&linphonec);
  return TRUE;
}


static void
voc_linphone_handler (voc_sid_t sid, tel_wifi_evt_t *in_evt)
{

  printf ("\n\t\tEnter the voc_linphone_handler!!!!!!!!!!!!!\n");
  tel_wifi_evt_t *evt;

  evt = g_new0 (tel_wifi_evt_t, 1);

  tel_wifi_cb_t cb;                     /**< Callback */
  void *user_data;                      /**< Private user data */
  tel_wifi_evt_data_t evt_data;                 /**< Event specific data */

  evt->evt_data = in_evt->evt_data;
  evt->evt_type = in_evt->evt_type;
  g_print ("evt->type=%d, sid=%d", evt->evt_type, sid);
  //if (in_evt->user_data!=NULL) {
  if (evt->evt_type == WIFI_EVT_LINE_CALL)
  {
    char *username = g_strdup (in_evt->user_data);
    char *delim = "\"<>";
    char *p = strtok (username, delim);

    evt->user_data = p;
    g_print ("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!the number is %s\n",
             in_evt->user_data);
    g_print ("!!!!!!!!!!!!!!!!!!111!!!!!!the number is %s\n", evt->user_data);
  }
  if (!evt->user_data)
  {
    char buffer[] = "Notknown";

    evt->user_data = g_strdup (buffer);
  }

  printf ("\n\t\tEnter the voc_linphone_handler:1!!!!!!!!!!!!!\n");
  if (!tel_wifi_evt_get (sid, &evt, 0))
  {
    g_print ("evt->type=%d", evt->evt_type);
    g_print ("evt->data=%d\n", evt->evt_data);
    if (evt->user_data)
      g_print ("!!!!!!!!!!!!!!!!!!!!!!222!!!!!!the number is %s\n",
               evt->user_data);

    tel_wifi_evt_call (sid, evt);
    if (evt->user_data)
      g_print ("!!!!!!!!!!!!!!!!!!!!!!333!!!!!!the number is %s\n",
               evt->user_data);


    tel_wifi_evt_release (sid, evt);
  }
}


tel_wifi_err_t
tapi_wifi_initialize ()
{
  wifi_register_evt_table =
    g_hash_table_new_full (tel_wifi_evt_table_hash,
                           tel_wifi_evt_table_equal, g_free,
                           tel_wifi_evt_table_destroy_value);
  /*snprintf(configfile_name, PATH_MAX, "%s/.linphonerc", "");*/
  snprintf (configfile_name, 256, "%s/etc/linphonerc", getenv ("V_ROOT"));

  /*linphone_core_disable_logs();*/
  linphone_core_enable_logs (stdout);

  /* Initialize linphone core */
  linphone_core_init (&linphonec, &linphonec_vtable, configfile_name, NULL);
  /* Initialize callback function */
  linphonec.cb_handler = voc_linphone_handler;

  /* open iterate timer */
  timeout_id = g_timeout_add (1000, (GSourceFunc) tapi_wifi_call_iterate, NULL);
  //timeout_id = g_timeout_add(1000,(GSourceFunc)linphonec_idle_call,NULL) ;

  return WIFI_ERR_NONE;

}

tel_wifi_err_t
tapi_wifi_finalize ()
{
  /* uninit linphone core */
  linphone_core_uninit (&linphonec);
  /* remove timer */
//        gtk_timeout_remove (timeout_id);
  g_source_remove (timeout_id);

}


tel_wifi_err_t
tel_wifi_call_dial (voc_sid_t sid, voc_lid_t lid,
                    voc_cid_t *cid, const char *remote)
{
  printf ("before lpc ipc\n");

  lpc_cmd_call (&linphonec, remote);
  printf ("after lpc ipc\n");

  return WIFI_ERR_NONE;
}

/**Triggers establishment of a connection between peers.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_wifi_err_t
tel_wifi_call_accept (voc_sid_t sid, voc_cid_t cid)
{
  lpc_cmd_answer (&linphonec, NULL);
  return WIFI_ERR_NONE;
}

/**Triggers termination of call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_wifi_err_t
tel_wifi_call_hungup (voc_sid_t sid, voc_cid_t cid)
{
  lpc_cmd_terminate (&linphonec, NULL);
  return WIFI_ERR_NONE;
}

/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_wifi_err_t
tel_wifi_call_send_dtmf (voc_sid_t sid, voc_cid_t cid, char *number)
{
  linphone_core_send_dtmf (&linphonec, *number);
  return WIFI_ERR_NONE;

}



/**Hold an active call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_wifi_err_t
tel_wifi_call_hold (voc_sid_t sid, voc_cid_t cid)
{
  if (-1 == linphone_core_on_hold_call (&linphonec, NULL))
  {
    printf ("Could not hold the call.\n");
    return WIFI_ERR_PROTOCAL;
  }
  return WIFI_ERR_NONE;
}




/**Unhold an held call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
tel_wifi_err_t
tel_wifi_call_unhold (voc_sid_t sid, voc_cid_t cid)
{
  if (-1 == linphone_core_off_hold_call (&linphonec, NULL))
  {
    printf ("Could not unhold the call.\n");
    return WIFI_ERR_PROTOCAL;
  }
  return WIFI_ERR_NONE;
}


/* 
vi:ts=2:nowrap:ai:expandtab
*/

