/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef	__LIPS_TAPI_WIFI_H
#define	__LIPS_TAPI_WIFI_H

#include "voctype.h"
#include "vocerror.h"
#include "linphone/linphonecore.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  typedef enum
  {
    WIFI_EVT_LINE,
    WIFI_EVT_LINE_SIGNAL,
    WIFI_EVT_LINE_CALL,
    WIFI_EVT_CALL_REMOTE_STATUS,
    WIFI_EVT_CALL_STATUS,
    WIFI_EVT_EXCEPTION
  } tel_wifi_evt_type_t;

  typedef struct tel_wifi_evt tel_wifi_evt_t;
  typedef void (*tel_wifi_cb_t) (voc_sid_t sid, tel_wifi_evt_t *evt);

  typedef struct
  {
    voc_sid_t sid;
    tel_wifi_cb_t cb;
    void *user_data;
  } tel_wifi_register_evt_t;

  typedef uint32_t tel_wifi_evt_data_t;
  
  struct tel_wifi_evt
  {
    tel_wifi_evt_type_t evt_type;               /**< Event Type */
    tel_wifi_cb_t cb;                   /**< Callback */
    void *user_data;                    /**< Private user data */
    tel_wifi_evt_data_t evt_data;               /**< Event specific data */
  };

  typedef enum
  {
    WIFI_ERR_NONE = 0,                 /**< No error */
    WIFI_ERR_INVALIDSESSION,           /**< This session is invalid */
    WIFI_ERR_NOTSUPPORT,               /**< This function is not supported */
    WIFI_ERR_NOLINK,                   /**< There is no link */
    WIFI_ERR_NOTREGISTERED,               /**< There is an unsolicited event, but not registered for current session */
    WIFI_ERR_DEVICE,                   /**< modem error */
    WIFI_ERR_PROTOCAL,                 /**< protocal error */
    WIFI_ERR_GENERAL,                      /**< General error */
    WIFI_ERR_NOEVENT,                              /**< wait for reply */
    WIFI_ERR_TIMEOUT,
    WIFI_ERR_UNKNOWN,                   /**< Unknown error */
    WIFI_ERR_DBUS
  } tel_wifi_err_t;

  tel_wifi_err_t tel_wifi_open_session (UInt32 *sid);
  tel_wifi_err_t tel_wifi_close_session (voc_sid_t sid);

  tel_wifi_err_t tapi_wifi_initialize ();
  tel_wifi_err_t tapi_wifi_finalize ();
  Boolean tel_wifi_if_registered (voc_sid_t sid, tel_wifi_evt_type_t evt_type);
  tel_wifi_err_t tel_wifi_evt_register (voc_sid_t sid,
                                        tel_wifi_evt_type_t evt_type,
                                        tel_wifi_cb_t cb, void *user_data);

  tel_wifi_err_t tel_wifi_call_hungup (voc_sid_t sid, voc_cid_t cid);
  tel_wifi_err_t tel_wifi_call_send_dtmf (voc_sid_t sid, voc_cid_t cid,
                                          char *number);
  tel_wifi_err_t tel_wifi_call_accept (voc_sid_t sid, voc_cid_t cid);
  tel_wifi_err_t tel_wifi_call_dial (voc_sid_t sid, voc_lid_t lid,
                                     voc_cid_t *cid, const char *remote);
  tel_wifi_err_t tel_wifi_call_hold (voc_sid_t sid, voc_cid_t cid);
  tel_wifi_err_t tel_wifi_call_unhold (voc_sid_t sid, voc_cid_t cid);




#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_TAPI_GSM_H */

/* 
vi:ts=2:nowrap:ai:expandtab
*/

