/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#define DEBUG

/******************************************************************************
  include
******************************************************************************/
#include <stdio.h>
#include <glib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

#include "vocevent.h"


/* structure to store registered event */
typedef struct
{
  voc_sid_t sid;
  voc_lid_t lid;
  voc_cid_t cid;
  voc_cb_t cb;
  void *user_data;
} register_evt_t;

static GHashTable *register_evt_table;
static register_evt_t *event_registered;


/************************************************************************/
/* Register hash table operations                           */
/************************************************************************/
/* 
 *	a function to create a hash value from a key.
 */
static guint
g_evt_table_hash (gconstpointer key)
{
  guint value = *(guint *) key;

  return value;
}

/* 
 *	a function to check two keys for equality.
 */
static gboolean
g_evt_table_equal (gconstpointer a, gconstpointer b)
{
  return *(guint *) a == *(guint *) b;
}

/* 
 *	a function to free the memory allocated for the 
 *  value used when removing the entry from the GHashTable.
 */
static void
g_evt_table_destroy_value (gpointer data)
{
  g_list_free ((GList *) data);
}

static Boolean
voc_if_registered (voc_sid_t sid, voc_lid_t lid, voc_cid_t cid,
                   voc_evt_type_t evt_type)
{

  g_print ("in %s\n", __FUNCTION__);

  GList *list = NULL;
  GList *nextem = NULL;
  register_evt_t *event;

  if (evt_type == VOC_EVT_LINE_CALL)
    cid = VALUE_NOEXIST;


  /* search sid in the event register list */
  list = (GList *) g_hash_table_lookup (register_evt_table, &evt_type);
  g_print ("in %s:1\n", __FUNCTION__);

  if (list != NULL)
  {
    nextem = list->next;        
    //nextem=list;
    while (nextem)
    {
      event = (register_evt_t *) nextem->data;
      g_print ("in %s:1.1: sid=%d,lid=%d,cid=%d,\n", __FUNCTION__,
               event->sid, event->lid, event->cid);

      if ((event->sid == sid) && (event->lid == lid) && (event->cid == cid))
      {
        g_print ("in %s:2\n", __FUNCTION__);
        g_print ("cb=0x%08x\n", (guint32) event->cb);
        g_print ("step 01: \n");



        /* store the event */
        event_registered = event;
        g_print ("step 02: \n");
        g_print ("step 1: cb=0x%08x\n", (guint32) event_registered->cb);

        /*remove the registered entry from the list */
        if (evt_type != VOC_EVT_LINE_CALL
            && evt_type != VOC_EVT_CALL_REMOTE_STATUS
            && evt_type != VOC_EVT_EXCEPTION)
        {

          if (g_list_length (list) == 1)
          {
            g_print ("Couldn't be here!\n\n\n");
            list = NULL;
            g_hash_table_remove (register_evt_table, &evt_type);
          }
          else
          {
            list = g_list_remove (list, event);
          }
          //g_free(event->user_data);
          //g_free(event);
        }

        return TRUE;
      }
      nextem = nextem->next;
    }
  }
  return FALSE;
}


/**
 *initialize VOC event module.
 */
voc_err_t
voc_evt_initialize ()
{
  /* Create the event hash table */
  register_evt_table =
    g_hash_table_new_full (g_evt_table_hash, g_evt_table_equal, g_free,
                           g_evt_table_destroy_value);
#ifdef DEBUG
  printf ("the original hash size is:%d\n",
          g_hash_table_size (register_evt_table));
#endif

  return VOC_ERR_NONE;
}




/*****************************************

  Event Operations

 ******************************************/
/**
 *Register a  event type.
 *@param sid session ID
 *@param evt event type (All event type)
 *@param filter filter type
 *@param cb event callback function
 *@param user_data the user defined data will be passed back in the callback function
 *@return register operation result, 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_evt_all_register (voc_sid_t sid, voc_lid_t lid, voc_cid_t cid,
                      voc_evt_type_t evt,
                      voc_evt_filter_t *filter, voc_cb_t cb, void *user_data)
{
  GList *list = NULL;
  register_evt_t *event;
  voc_evt_type_t *key = NULL;

  key = g_new0 (voc_evt_type_t, 1);
  *key = evt;
  event = (register_evt_t *) malloc (sizeof (register_evt_t));
  event->sid = sid;
  event->lid = lid;
  event->cid = cid;
  event->cb = cb;
  event->user_data = user_data;
  /* first check if the hash key for event type is used,
     if used, add the event in the GPtrArray of that key,
     otherwise insert a new key-value pair for that event type */
  list = (GList *) g_hash_table_lookup (register_evt_table, key);
  if (list == NULL)
  {

    list = g_list_alloc ();     //0613
    list = g_list_append (list, event);
    g_hash_table_insert (register_evt_table, key, list);
  }
  else
  {
    g_list_append (list, event);
  }

#ifdef DEBUG
  printf ("after regis, the hash size is:%d,type=%d,sid=%d,lid=%d,cid=%d\n",
          g_hash_table_size (register_evt_table), evt, sid, lid, cid);
  printf ("and the list size is:%d\n", g_list_length (list));
#endif

  /* 
     Do not need to free list here.
     The list is used as value in the key-value pair of hash-table
     which wil be destroyed in voc_evt_unregister()
     when removing the pair from the hash-table.
     By that time, the memory in the list is also freed.
   */

  return VOC_ERR_NONE;
}

/**
 *@brief	Register for a session-wide event.
 *@param sid session ID
 *@param evt event type (VOC_EVT_LINE)
 *@param cb event callback function
 *@param user_data the user defined data witch will passed back in the callback function
 *@return 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_evt_register (voc_sid_t sid, voc_evt_type_t evt,
                  voc_evt_filter_t *filter, voc_cb_t cb, void *user_data)
{
  return voc_evt_all_register (sid, VALUE_NOEXIST, VALUE_NOEXIST, evt,
                               filter, cb, user_data);
}

/**
 *@brief	Register for an event on a line.
 *@param sid session ID
 *@param lid line ID
 *@param evt event type (VOC_EVT_LINE_STATUS or VOC_EVT_LINE_CALL)
 *@param cb event callback function
 *@param user_data the user defined data witch will passed back in the callback function
 *@return 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_line_evt_register (voc_sid_t sid, voc_lid_t lid,
                       voc_evt_type_t evt,
                       voc_evt_filter_t *filter, voc_cb_t cb, void *user_data)
{
  return voc_evt_all_register (sid, lid, VALUE_NOEXIST, evt, filter, cb,
                               user_data);
}


/**
 *@brief	Register for an event for a call.
 *@param sid session ID
 *@param cid call ID
 *@param evt event type (VOC_EVT_CALL_STATUS)
 *@param cb event callback function
 *@param user_data the user defined data witch will passed back in the callback function
 *@return 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_call_evt_register (voc_sid_t sid, voc_cid_t cid,
                       voc_evt_type_t evt,
                       voc_evt_filter_t *filter, voc_cb_t cb, void *user_data)
{
  return voc_evt_all_register (sid, VALUE_NOEXIST, cid, evt, filter, cb,
                               user_data);
}


/**
 *Deregister an event type.
 *@param sid session ID
 *@param evt event type
 *@return deregister operation result
 */
voc_err_t
voc_evt_all_unregister (voc_sid_t sid, voc_lid_t lid,
                        voc_cid_t cid, voc_evt_type_t evt)
{
  GList *list = NULL;
  GList *nextem = NULL;
  register_evt_t *event;

  list = (GList *) g_hash_table_lookup (register_evt_table, &evt);
  /* now we remove the event with sid from the list */
  if (list != NULL)
  {
    nextem = list->next;       
    /*nextem=list; */
    while (nextem)
    {
      event = (register_evt_t *) nextem->data;
      if ((event->sid == sid) && (event->lid == lid) && (event->cid == cid))
      {
        if (g_list_length (list) == 1)
        {
          list = NULL;
          g_hash_table_remove (register_evt_table, &evt);
        }
        else
        {
          list = g_list_remove (list, event);
        }
        g_free (event->user_data);
        g_free (event);

#ifdef DEBUG
        printf ("after unregis, the hash size is:%d\n",
                g_hash_table_size (register_evt_table));
        printf ("after unregis, the list size is:%d\n", g_list_length (list));
#endif
        /*Not necessary, has been freed at g_list_remove
        g_list_free(nextem);*/
        return VOC_ERR_NONE;
      }
      nextem = nextem->next;
    }
  }

  /* Can not do it!if so, the next list element would be modified! 
     g_list_free(list);
   */
  /* Not necessary!
     if (nextem)
     g_list_free(nextem);
   */
  return VOC_ERR_GENERAL;
}

/**
 *@brief	Cease registration for a specific session-wise event.
 *@param sid session ID
 *@param evt event type
 *@return 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_evt_unregister (voc_sid_t sid, voc_evt_type_t evt)
{
  voc_evt_all_unregister (sid, VALUE_NOEXIST, VALUE_NOEXIST, evt);
  return VOC_ERR_NONE;
}

/**
 *@brief	Cease registration for an event on a prticular line.
 *@param sid session ID
 *@param lid line id
 *@param evt event type
 *@return 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_line_evt_unregister (voc_sid_t sid, voc_lid_t lid, voc_evt_type_t evt)
{
  voc_evt_all_unregister (sid, lid, VALUE_NOEXIST, evt);
  return VOC_ERR_NONE;
}


/**
 *@brief	Cease registration for an event for a particular call.
 *@param sid session ID
 *@param cid call id
 *@param evt event type
 *@return 0 if successfull; otherwise a negative error code
 */
voc_err_t
voc_call_evt_unregister (voc_sid_t sid, voc_cid_t cid, voc_evt_type_t evt)
{
  voc_evt_all_unregister (sid, VALUE_NOEXIST, cid, evt);
  return VOC_ERR_NONE;
}


/**
 *Delivers an event to the client by calling the callback function specified
 *through the corresponding event_register request. The callback function is called in the context of the event_process request.
 *@param voc_sid_t session ID
 *@param sid the timeout of the event pending
 *@return the result of the operation
 */
//voc_err_t voc_evt_process (voc_sid_t sid, int timeout)

voc_err_t
voc_evt_process (voc_sid_t sid, int timeout)
{
  voc_err_t result = 0;
  voc_evt_t *evt = NULL;

  result = voc_evt_get (sid, &evt, timeout);
  if (result != 0)
    return result;
  voc_evt_call (sid, evt);
  voc_evt_release (sid, evt);
  return VOC_ERR_NONE;
}

/**
 *Package the event from DBUS
 *@param sid session ID
 *@param evt the event to pend
 *@param timeout the timeout of the event pending
 *@return the result of the operation
 */
voc_err_t
voc_evt_get (voc_sid_t sid, voc_evt_t **evt, int timeout)
{
  g_print ("in %s\n", __FUNCTION__);
  voc_evt_type_t event_in;

  voc_lid_t lid = VALUE_NOEXIST;

  voc_cid_t cid = VALUE_NOEXIST;

  event_in = (*evt)->evt_type;

  if (event_in == VOC_EVT_LINE_SIGNAL)
  {
    lid = (*evt)->evt_data;
  }
  else if (event_in == VOC_EVT_CALL_STATUS)
  {
    cid = (*evt)->evt_data;
  }
  else if (event_in == VOC_EVT_LINE_CALL)
  {
    /*0613, now lid is saved into evt_data, and cid is achieved and saved in cb_psevt
    lid = (*evt)->evt_data;
    cid=VALUE_NOEXIST;*/
    lid = 1;
    cid = (*evt)->evt_data;

  }

  g_print ("sid=%d, lid=%d,cid=%d,event_in=%d\n", sid, lid, cid, event_in);
  if (!voc_if_registered (sid, lid, cid, event_in))
  {
    return VOC_ERR_NOTREGISTERED;
  }

  /* 
   *     Process message
   */
  /*(*evt)->evt_type = event_received; //Has been sent in dbus.if*/
  g_print ("in %s\n", __FUNCTION__);

  g_print ("cb=0x%08x\n", (guint32) event_registered->cb);

  (*evt)->cb = event_registered->cb;

  g_print ("in %s5\n", __FUNCTION__);

  g_print ("cb=0x%08x\n", (guint32) (*evt)->cb);

  if (event_in != VOC_EVT_LINE_CALL)
    (*evt)->user_data = event_registered->user_data;
  //g_free(event->user_data);
  //g_free(event_registered);


  g_print ("in %s6\n", __FUNCTION__);

#ifdef DEBUG
  printf ("type=%d, lid=%d, cid=%d\n", (*evt)->evt_type, lid, cid);
#endif

  return VOC_ERR_NONE;
}

/**
 *Invokes the callback function contained in the descriptor.
 *@param sid session ID
 *@param evt the event in the operation
 *@return the result of the operation
 */
voc_err_t
voc_evt_call (voc_sid_t sid, voc_evt_t *evt)
{
  g_print ("in upper voc_evt_handler\n");
  g_print ("cb=0x%08x\n", (guint32) evt->cb);
  voc_cb_t cb;

  cb = (voc_cb_t) evt->cb;
  g_print ("cb=0x%08x\n", (guint32) cb);

  (*cb) (sid, evt);

  g_print ("after upper voc_evt_handler\n");
  return VOC_ERR_NONE;
}

/**
 *Release all resources allocated for the event.
 *@param sid session ID
 *@param evt the event in the operation
 *@return the result of the operation
 */
voc_err_t
voc_evt_release (voc_sid_t sid, voc_evt_t *evt)
{
  if (!evt)
    return VOC_ERR_NONE;
  /* First free the event related data */
  //g_free(evt->evt_data);

  /* Then free the event itself */
  g_free (evt);
  return VOC_ERR_NONE;
}

/* 
vi:ts=2:nowrap:ai:expandtab
*/

