/* 	libvocenabler: - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	         YU Yijun <yijun.yu@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef	__LIPS_VOC_SESSION_GSM_H
#define	__LIPS_VOC_SESSION_GSM_H

/******************************************************************************
  include
******************************************************************************/
#include <stdio.h>
#include <glib.h>

#include "vocsession-priv.h"

#include "tapi-gsm.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */




/**Initialize gsm module
 *
 *@param info 	the pointer to the VocInfo structure
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t _voc_funcs_init_gsm (VocInfo *info);

/**Initialize gsm funciton
 *
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t voc_gsm_initialize ();
  
/**Finalize gsm funciton
 *
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t voc_gsm_finalize ();

/**Open gsm session
 *
 *@param sid 	the session id
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t voc_gsm_open_session (voc_sid_t *sid);

/**Close gsm session
 *
 *@param sid 	the session id
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t voc_gsm_close_session (voc_sid_t sid);


/**Create a new call object
 *
 *@param sid 	the Session ID
 *@param lid 	the Line ID
 *@param callp	a pointer to the location where, in the case of success, 
 *		the Call ID of the created call object will be returned.
 *
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t voc_call_create (voc_sid_t sid, voc_lid_t lid, voc_cid_t *cid);


/**Release a call object.
 *
 *@param sid 	the Session ID
 *@param cid the target Call ID. The implmentation can destroy the object when the object has been released by 
 *			all clients that got its id via a voc_call_create() call or a VOC_EVT_LINE_CALL event.
 *
 *@return 0 if successfull; otherwise a negative error code newly created voice call object
 */
  voc_err_t voc_call_release (voc_sid_t sid, voc_cid_t cid);



/**Return the state of a call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *@param statp	a pointer to the location where, in the case of success, 
 *		the current state of the call will be returned.
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_call_get_state (voc_sid_t sid, voc_cid_t cid,
                                voc_call_state_t *statep);

/**Return the line of a call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *@param statp	a pointer to the location where, in the case of success, 
 *		the lid of the call's line will be returned.
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_call_get_line (voc_sid_t sid, voc_cid_t cid, voc_lid_t *lidp);

/**
 *@brief	create and initiate a call to a remote peer.
 *@param sid 	the Session ID
 *@param lid	the target line id. a call will be created from this line.
 *@param cid	the ID of newly created outgoing Call
 *@param remote designate the remote peer. Could be a number of an URI.
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_dial (voc_sid_t sid, voc_lid_t lid, voc_cid_t *cid,
                          const char *remote);

/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_send_dtmf (voc_sid_t sid, voc_cid_t cid, char *number);


/**
 *@brief	set mute/unmute
 *@param sid the Session ID
 *@param is_mute	 1 mean set mute, 0 means unmute
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_mute (voc_sid_t sid, UInt32 is_mute);

/**
 *@brief	switch between hold and unhold
 *@param sid the Session ID
 *@param cid the call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_onhold (voc_sid_t sid, voc_cid_t cid);


/**Triggers establishment of a connection between peers.
 *
 *@param sid 	the Session ID
 *@param cid	the Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_accept (voc_sid_t sid, voc_cid_t cid);



/**
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_hungup (voc_sid_t sid, voc_cid_t cid);

/**
 *Triggers termination of all calls
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_hungup_all (voc_sid_t sid, voc_cid_t cid);



/**Triggers cancelation of an active call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_hungup_active (voc_sid_t sid, voc_cid_t cid);

/**Triggers operation of accepting an incoming call, at the same time, terminating the hold call.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_replace_hold (voc_sid_t sid, voc_cid_t cid);

/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_hungup_outgoing (voc_sid_t sid, voc_cid_t cid);



/**Triggers termination of the call on hold.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_hungup_held (voc_sid_t sid, voc_cid_t cid);



/**Triggers operation of put all calls in conference.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_conference (voc_sid_t sid, voc_cid_t cid);

/**Triggers operation of make all calls exit conference.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_exit_conference (voc_sid_t sid, voc_cid_t cid);

/**
 *@brief	get the start time of a call
 *@param sid the Session ID
 *@param cid	the target incoming Call ID
 *@param start pointer to the time structure which contains the start time
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_get_start_time (voc_sid_t sid, voc_cid_t cid,
                                    voc_time_t *start);

/**
 *@brief	get the end time of a call
 *@param sid the Session ID
 *@param cid	the target incoming Call ID
 *@param end pointer to the time structure which contains the end time
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_get_end_time (voc_sid_t sid, voc_cid_t cid,
                                  voc_time_t *end);

/**
 *@brief	Return the remote number of a call.
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *@param remote	returned remote phone number.
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_gsm_get_remote (voc_sid_t sid, voc_cid_t cid, char **remote);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */


#endif                          /* __LIPS_VOC_SESSION_GSM_H */


/* 
vi:ts=2:nowrap:ai:expandtab
*/

