/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#define DEBUG

/******************************************************************************
  include
******************************************************************************/
#include <stdio.h>
#include <glib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include "vocsession-priv.h"

#include "vocsession-mul.h"
#include "vocsession-gsm.h"
#include "vocsession-wifi.h"


#include "vocevent.h"
#include "tapi-gsm.h"

static UInt32 g_lid = 0;
static UInt32 g_cid = 0;

static GPtrArray *vsessions = NULL;
static GHashTable *vsession_maps = NULL;


static VocInfo *
voc_info_new (void)
{
  VocInfo *info = NULL;

  info = g_new0 (VocInfo, 1);
  if (info == NULL)
  {
    return NULL;
  }

  return info;
}


static void
voc_info_free (VocInfo *info)
{
  if (info == NULL)
  {
    return;
  }

  if (info->funcs)
  {
    g_free (info->funcs);
  }

  g_free (info);

  return;
}


static void
voc_info_print (const VocInfo *info)
{
  g_print ("VocInfo[0x%08x]\n", (guint32) info);
  g_print ("|ID     : %d\n", info->id);
  g_print ("|TYPE   : %d\n", info->type);
  g_print ("`FUNCS  : 0x%08x\n", (guint32) info->funcs);

  if (info->funcs)
  {
    g_print (" |voc_initialize                : 0x%08x\n",
             (guint32) info->funcs->voc_initialize);
    g_print (" |voc_finalize                 : 0x%08x\n",
             (guint32) info->funcs->voc_finalize);
    g_print (" |voc_dial                   : 0x%08x\n",
             (guint32) info->funcs->voc_dial);
    g_print (" |voc_send_dtmf             : 0x%08x\n",
             (guint32) info->funcs->voc_send_dtmf);
    g_print (" |voc_mute                : 0x%08x\n",
             (guint32) info->funcs->voc_mute);
    g_print (" |voc_onhold                 : 0x%08x\n",
             (guint32) info->funcs->voc_onhold);
    g_print (" |voc_unhold                 : 0x%08x\n",
             (guint32) info->funcs->voc_unhold);
    g_print (" |voc_accept                : 0x%08x\n",
             (guint32) info->funcs->voc_accept);
    g_print (" |voc_hungup            : 0x%08x\n",
             (guint32) info->funcs->voc_hungup);
    g_print (" |voc_hungup_all                : 0x%08x\n",
             (guint32) info->funcs->voc_hungup_all);
    g_print (" |voc_hungup_active            : 0x%08x\n",
             (guint32) info->funcs->voc_hungup_active);
    g_print (" |voc_replace_hold    : 0x%08x\n",
             (guint32) info->funcs->voc_replace_hold);
    g_print (" |voc_hungup_outgoing           : 0x%08x\n",
             (guint32) info->funcs->voc_hungup_outgoing);
    g_print (" |voc_hungup_held               : 0x%08x\n",
             (guint32) info->funcs->voc_hungup_held);
    g_print (" |voc_conference                : 0x%08x\n",
             (guint32) info->funcs->voc_conference);
    g_print (" |voc_exit_conference              : 0x%08x\n",
             (guint32) info->funcs->voc_exit_conference);

  }

  return;
}

static voc_err_t
voc_add_line (voc_linetype_t type, voc_lid_t *lidp)
{

  *lidp = g_lid++;

  VocInfo *info = voc_info_new ();

  info->id = *lidp;
  info->type = type;
  //info->source  = g_strdup(argv[2]);
  //info->ref                     = 0;
  info->funcs = NULL;
  g_ptr_array_add (vsessions, (gpointer) info);

}


voc_err_t
voc_initialize ()
{

  /* Create the event hash table */

  if (voc_evt_initialize ())
  {
    g_print ("Error:Could not initialize the voc module!\n");
    return VOC_ERR_GENERAL;
  }

  g_print ("%s():\n", __FUNCTION__);
  vsessions = g_ptr_array_new ();
  vsession_maps =
    g_hash_table_new_full (g_int_hash, g_int_equal, g_free, g_free);

  voc_lid_t lid;

  voc_add_line (VOC_GSM, &lid);
  voc_add_line (VOC_WIFI, &lid);
  g_print ("the length of the vsessions table is %d:\n", vsessions->len);
  int i;

  for (i = 0; i < vsessions->len; i++)
  {
    VocInfo *info = (VocInfo *) g_ptr_array_index (vsessions, i);

    switch (info->type)
    {
      case VOC_GSM:
        g_print ("in:%s: ini VOC_GSM\n", __FUNCTION__);
        _voc_funcs_init_gsm (info);
        if (info->funcs)
        {
          if (info->funcs->voc_initialize (info) != VOC_ERR_NONE)
          {
            return VOC_ERR_GENERAL;
          }
        }
        break;

      case VOC_WIFI:
        g_print ("in:%s: ini VOC_WIFI\n", __FUNCTION__);
        _voc_funcs_init_wifi (info);
        if (info->funcs)
        {
          if (info->funcs->voc_initialize (info) != VOC_ERR_NONE)
          {
            return VOC_ERR_GENERAL;
          }
        }
        break;

      default:
        g_print ("%s(): type [%d] is unsupported.\n", __FUNCTION__, info->type);
        break;
    }

  }

  return VOC_ERR_NONE;

}

voc_err_t
voc_finalize (void)
{
  int count, i;

  g_print ("%s():\n", __FUNCTION__);

  g_hash_table_destroy (vsession_maps);

  count = vsessions->len;
  for (i = 0; i < count; i++)
  {
    VocInfo *info = NULL;

    info = g_ptr_array_remove_index_fast (vsessions, 0);
    if (info->funcs)
    {
      info->funcs->voc_finalize (info);
    }

    voc_info_free (info);
  }
  g_ptr_array_free (vsessions, TRUE);
}

/**Open a session with the voice call enabler
 *
 *@param sidp	a pointer to the location where in the case of success,
 *		a Session ID will be returned 
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_open_session (voc_lid_t index, voc_sid_t *sid)
{
  switch (index)
  {
    case VOC_GSM:
      voc_gsm_open_session (sid);
      break;
    case VOC_WIFI:
      voc_wifi_open_session (sid);
      break;
    default:
      g_print ("error: %s(): type [%d] is unsupported.\n", __FUNCTION__, index);
      break;
  }

  VocInfo *info = NULL;
  guint32 *svalue = NULL;
  guint32 *ivalue = NULL;

  if (sid == NULL)
  {
    return VOC_ERR_GENERAL;
  }

  if (index < 0 || index >= vsessions->len)
  {
    g_print ("%s(): [%d] is out of range [0 ... %d].\n", __FUNCTION__,
             index, (vsessions->len - 1));
    return VOC_ERR_TOO_BIG;
  }

  info = (VocInfo *) g_ptr_array_index (vsessions, index);

  svalue = g_new0 (guint32, 1);
  *svalue = *sid;
  ivalue = g_new (guint32, 1);
  *ivalue = index;

  g_print ("%s(): sid = %d, index = %d\n", __FUNCTION__, *sid, *ivalue);

  g_hash_table_replace (vsession_maps, (gpointer) svalue, (gpointer) ivalue);

  return VOC_ERR_NONE;

}



/**Close a session with the voice call enabler
 *
 *@param sid 	the target Session ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_close_session (voc_sid_t sid)
{
  //JD1106
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    VocInfo *info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    g_hash_table_remove (vsession_maps, &sid);
  }



  switch (*index)
  {
    case VOC_GSM:
      voc_gsm_close_session (sid);
      break;
    case VOC_WIFI:
      voc_wifi_close_session (sid);
      break;
    default:
      g_print ("error: %s(): type [%d] is unsupported.\n", __FUNCTION__, index);
      break;
  }

  return VOC_ERR_CARRIER;
}


/*****************************************

  Line Operations

 ******************************************/
/**Connect to a line
 *
 *@param sid 	the Session ID
 *@param i	the target line index. All lines are indexed from 0 to n.
 *@param lid	a pointer to the location where, in the case of success, 
 *		the target Line ID will be returned.
 *@return 0 if successfull; 
 *	VOC_ERR_NOT_EXIST
 *		no line with the specified index exist 
 *		but lines with higher indexes may still exist
 *	VOC_ERR_TOO_BIG
 *		no lines exist with any index equal to or higher than the specified @param i
 */
voc_err_t
voc_line_connect (voc_sid_t sid, uint32_t i, voc_lid_t *lidp)
{
  /*To do:
  //To be fixed when the device manager is ready
  //THE FOLLOWING CODES ARE ONLY FOR TEST
  */
  //* lidp =g_lid++ ;
  *lidp = 1;

  if (i < 1)
  {
    return VOC_ERR_NONE;
  }
  else
    return VOC_ERR_TOO_BIG;
}

/**Disconnect from a line
 *
 *@param sid 	the Session ID
 *@param lid	the target Line ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */

 /**really register a line to the network. Before calling this func, app
 should call voc_line_is_registered first to check whether or not this line
 is already registered.
 */

voc_err_t
voc_line_disconnect (voc_sid_t sid, voc_lid_t lid)
{
  return VOC_ERR_NONE;
}

/**Return a printable name of a line.
 *
 *@param sid 	the Session ID
 *@param lid	the target Line ID
 *@param namep	a pointer to the location where, in the case of success, 
 *		a pointer to the string containing the name of the line will be returned.
 *		Note that it's the responsability of the caller to free the string's memory eventually.
 *
 *@return 0 if successfull; otherwise a negative error code 
 */

voc_err_t
voc_line_get_name (voc_sid_t sid, voc_lid_t lid, char **namep)
{
//To do:
  //To be fixed when the device manager is ready
  //THE FOLLOWING CODES ARE ONLY FOR TEST
  return VOC_ERR_NONE;
}

/**Return a short printable decription of a line.
 *
 *@param sid 	the Session ID
 *@param lid	the target Line ID
 *@param descp	a pointer to the location where, in the case of success, 
 *		a pointer to the string containing the description of the line will be returned.
 *		Note that it's the responsability of the caller to free the string's memory eventually.
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_line_get_description (voc_sid_t sid, voc_lid_t lid, char **descp)
{
//To do:
  //To be fixed when the device manager is ready
  //THE FOLLOWING CODES ARE ONLY FOR TEST
  return VOC_ERR_NONE;
}


/**querying line signal
*/
voc_err_t
voc_line_get_signal (voc_sid_t sid, voc_lid_t lid, voc_line_signal_t *signal)
{
//To do:
  //To be fixed when the device manager is ready
  //THE FOLLOWING CODES ARE ONLY FOR TEST
  return VOC_ERR_NONE;
}


voc_err_t
voc_line_get_local_address (voc_sid_t sid, voc_lid_t lid, char **local)
{
//To do:
  //To be fixed when the device manager is ready
  //THE FOLLOWING CODES ARE ONLY FOR TEST
  return VOC_ERR_NONE;
}

/**
 *@brief	create and initiate a call to a remote peer.
 *@param sid 	the Session ID
 *@param lid	the target line id. a call will be created from this line.
 *@param cid	the ID of newly created outgoing Call
 *@param remote designate the remote peer. Could be a number of an URI.
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_dial (voc_sid_t sid, voc_lid_t lid, voc_cid_t *cid, const char *remote)
{
  //JD0611
  VocInfo *info = NULL;
  guint32 *index = NULL;

  g_print ("in %s, sid=%d\n", __FUNCTION__, sid);
  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);

  g_print ("in %s, sid=%d, index=%d\n", __FUNCTION__, sid, *index);
  g_print ("dialgcb=0x%08x\n", (guint32) voc_gsm_dial);
  g_print ("dialwificb=0x%08x\n", (guint32) voc_wifi_dial);

  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);
    g_print ("type=%d\n", info->type);
    g_print ("1cb=0x%08x\n", (guint32) info->funcs);
    g_print ("2cb=0x%08x\n", (guint32) info->funcs->voc_dial);

    if (info && info->funcs)
    {
      return (info->funcs->voc_dial (sid, lid, cid, remote));
    }
  }

  return VOC_ERR_GENERAL;
}



/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_send_dtmf (voc_sid_t sid, voc_cid_t cid, char *number)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_send_dtmf (sid, cid, number));
    }
  }

  return VOC_ERR_GENERAL;
}


/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_mute (voc_sid_t sid, UInt32 is_mute)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_mute (sid, is_mute));
    }
  }

  return VOC_ERR_GENERAL;
}


/**Triggers transition of a connected call in the onhold state.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_onhold (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_onhold (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}




/**Triggers transition of a connected call in the onhold state.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_unhold (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_unhold (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}


/**Triggers establishment of a connection between peers.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_accept (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_accept (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}



/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_hungup (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_hungup (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}


voc_err_t
voc_hungup_all (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_hungup_all (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}



/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_hungup_active (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_hungup_active (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}



/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_replace_hold (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_replace_hold (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}

/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_hungup_outgoing (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_hungup_outgoing (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}



/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_hungup_held (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_hungup_held (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}



/**Triggers transition of a connected call in the onhold state.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_conference (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_conference (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}


/**Triggers transition of a connected call in the onhold state.
 *
 *@param sid 	the Session ID
 *@param cid	the target incoming Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_exit_conference (voc_sid_t sid, voc_cid_t cid)
{
  VocInfo *info = NULL;
  guint32 *index = NULL;

  index = (guint32 *) g_hash_table_lookup (vsession_maps, &sid);
  if (index)
  {
    info = (VocInfo *) g_ptr_array_index (vsessions, *index);

    if (info && info->funcs)
    {
      return (info->funcs->voc_exit_conference (sid, cid));
    }
  }

  return VOC_ERR_GENERAL;
}


/**
 *@brief	list info of each call records
 */

voc_err_t
voc_get_start_time (voc_sid_t sid, voc_cid_t cid, voc_time_t *start)
{
  char timebuf[40];
  struct tm *tm;

  time (start);
  tm = localtime (start);
#ifdef DEBUG
//      printf("%s",ctime(start));
  strftime (timebuf, 40, "%X", tm);
  printf ("%s", timebuf);
#endif

  return VOC_ERR_NONE;
}


voc_err_t
voc_get_end_time (voc_sid_t sid, voc_cid_t cid, voc_time_t *end)
{
  char timebuf[40];
  struct tm *tm;

  time (end);
  tm = localtime (end);
#ifdef DEBUG
  strftime (timebuf, 40, "%X", tm);
  printf ("%s", timebuf);
#endif

  return VOC_ERR_NONE;
}

/**
 *@brief	Return the remote number of a call.
 *@param sid 	the Session ID
 *@param cid	the target Call ID
 *@param remote	returned remote phone number.
 *@return 0 if successfull; otherwise a negative error code 
 */
voc_err_t
voc_get_remote (voc_sid_t sid, voc_cid_t cid, char **remote)
{
  return VOC_ERR_NONE;
}


/* 
vi:ts=2:nowrap:ai:expandtab
*/

