/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef	__VOC_PRIVATE_H
#define __VOC_PRIVATE_H

#include <glib.h>

#include <voctype.h>
#include <vocerror.h>

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  typedef struct _VocInfo VocInfo;
  typedef struct _VocFuncs VocFuncs;

  struct _VocInfo
  {
    gint32 id;
    gint32 type;
    VocFuncs *funcs;
  };

  struct _VocFuncs
  {
    voc_err_t (*voc_initialize) (VocInfo *info);
    voc_err_t (*voc_finalize) (VocInfo *info);

    voc_err_t (*voc_dial) (voc_sid_t sid, voc_lid_t lid, voc_cid_t *cid,
                           const char *remote);
    voc_err_t (*voc_send_dtmf) (voc_sid_t sid, voc_cid_t cid, char *number);
    voc_err_t (*voc_mute) (voc_sid_t sid, UInt32 is_mute);
    voc_err_t (*voc_onhold) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_unhold) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_accept) (voc_sid_t sid, voc_cid_t cid);

    voc_err_t (*voc_hungup) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_hungup_all) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_hungup_active) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_replace_hold) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_hungup_outgoing) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_hungup_held) (voc_sid_t sid, voc_cid_t cid);

    voc_err_t (*voc_conference) (voc_sid_t sid, voc_cid_t cid);
    voc_err_t (*voc_exit_conference) (voc_sid_t sid, voc_cid_t cid);

  } _VocFuncs;


#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __VOC_PRIVATE_H */

/* 
vi:ts=2:nowrap:ai:expandtab
*/

