/* 	libvocenabler - API definition of the voice call service
 *
 *	Authors: JIANG Dan <dan.jiang@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 *	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */


#define DEBUG
#ifdef __cplusplus
extern "C"
{
#endif

/* ==================================================================================================
  DESCRIPTION
  ====================================================================================================*/



/******************************************************************************
  include
******************************************************************************/
#include <stdio.h>
#include <glib.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#include "vocsession-priv.h"
#include "vocsession-mul.h"

#include "vocevent.h"

#include "vocsession-wifi.h"
#include "tapi-wifi.h"



  static void voc_evt_handler (voc_sid_t sid_dbus, tel_wifi_evt_t *in_evt);


  voc_err_t _voc_funcs_init_wifi (VocInfo *info)
  {
    g_assert (info);
    g_assert (info->funcs == NULL);

    g_print ("%s():\n", __FUNCTION__);

    info->funcs = g_new0 (VocFuncs, 1);
    if (info->funcs == NULL)
    {
      return VOC_ERR_GENERAL;
    }


    info->funcs->voc_initialize = voc_wifi_initialize;

    //info->funcs->voc_finalize                     = voc_wifi_finalize;

    info->funcs->voc_dial = voc_wifi_dial;
    info->funcs->voc_send_dtmf = voc_wifi_send_dtmf;
    info->funcs->voc_mute = voc_wifi_mute;
    info->funcs->voc_onhold = voc_wifi_onhold;
    info->funcs->voc_unhold = voc_wifi_unhold;
    info->funcs->voc_accept = voc_wifi_accept;

    info->funcs->voc_hungup = voc_wifi_hungup;
    info->funcs->voc_hungup_all = voc_wifi_hungup_all;
    info->funcs->voc_hungup_active = voc_wifi_hungup_active;
    info->funcs->voc_replace_hold = voc_wifi_replace_hold;
    info->funcs->voc_hungup_outgoing = voc_wifi_hungup_outgoing;
    info->funcs->voc_hungup_held = voc_wifi_hungup_held;

    info->funcs->voc_conference = voc_wifi_conference;
    info->funcs->voc_exit_conference = voc_wifi_exit_conference;

    return VOC_ERR_NONE;
  }



  voc_err_t voc_wifi_initialize ()
  {
    tapi_wifi_initialize ();
  }


  voc_err_t voc_wifi_finalize ()
  {
    tapi_wifi_finalize ();
    return VOC_ERR_NONE;
  }

  voc_err_t voc_wifi_open_session (voc_sid_t *sid)
  {
    tel_wifi_open_session (sid);
    tel_wifi_evt_register (*sid, WIFI_EVT_LINE_CALL, voc_evt_handler, NULL);
    tel_wifi_evt_register (*sid, WIFI_EVT_CALL_STATUS, voc_evt_handler, NULL);
    tel_wifi_evt_register (*sid, WIFI_EVT_CALL_REMOTE_STATUS, voc_evt_handler,
                           NULL);
    tel_wifi_evt_register (*sid, WIFI_EVT_EXCEPTION, voc_evt_handler, NULL);

    g_print ("after 1 regis!\n");

  }

  voc_err_t voc_wifi_close_session (voc_sid_t sid)
  {
    tel_wifi_close_session (sid);
  }

  void voc_evt_handler (voc_sid_t sid_dbus, tel_wifi_evt_t *in_evt)
  {
    g_print ("in voc_evt_handler\n");
    g_print ("sid=%d,type=%d\n", sid_dbus, in_evt->evt_type);
    voc_evt_t *evt = g_new0 (voc_evt_t, 1);

    evt->evt_type = in_evt->evt_type;
    //evt->user_data = in_evt->user_data;
    evt->evt_data = in_evt->evt_data;
    if (in_evt->user_data)
    {
      g_print ("in voc_evt_handler, the call number is %s", in_evt->user_data);
      evt->user_data = in_evt->user_data;
    }

    if (!voc_evt_get (sid_dbus, &evt, 0))
    {
      g_print ("evt->type=%d", evt->evt_type);
      g_print ("evt->data=%d\n", evt->evt_data);
      voc_evt_call (sid_dbus, evt);
      g_print ("after call!");
      voc_evt_release (sid_dbus, evt);
    }
  }


/**
 *@brief	create and initiate a call to a remote peer.
 *@param sid 	the Session ID
 *@param lid	the target line id. a call will be created from this line.
 *@param cid	the ID of newly created outgoing Call
 *@param remote designate the remote peer. Could be a number of an URI.
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_wifi_dial (voc_sid_t sid, voc_lid_t lid, voc_cid_t *cid,
                           const char *remote)
  {
    printf ("before tel ipc\n");
    *cid = 60001;               //jd

    if (tel_wifi_call_dial (sid, lid, cid, remote) == WIFI_ERR_NONE)
    {
      printf ("after tel ipc\n");

      return VOC_ERR_NONE;
    }
    else
      return VOC_ERR_CARRIER;
  }





/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_wifi_send_dtmf (voc_sid_t sid, voc_cid_t cid, char *number)
  {
    if (tel_wifi_call_send_dtmf (sid, cid, number) == WIFI_ERR_NONE)
      return VOC_ERR_NONE;
    else
      return VOC_ERR_CARRIER;
  }


/**
 *@brief	send dtmf to a remote peer.
 *@param sid the Session ID
 *@param cid	 the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_wifi_mute (voc_sid_t sid, UInt32 is_mute)
  {
  }


  voc_err_t voc_wifi_onhold (voc_sid_t sid, voc_cid_t cid)
  {
    if (tel_wifi_call_hold (sid, cid) == WIFI_ERR_NONE)
      return VOC_ERR_NONE;
    else
      return VOC_ERR_CARRIER;
  }

  voc_err_t voc_wifi_unhold (voc_sid_t sid, voc_cid_t cid)
  {
    if (tel_wifi_call_unhold (sid, cid) == WIFI_ERR_NONE)
      return VOC_ERR_NONE;
    else
      return VOC_ERR_CARRIER;
  }


/**Triggers establishment of a connection between peers.
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_wifi_accept (voc_sid_t sid, voc_cid_t cid)
  {
    if (tel_wifi_call_accept (sid, cid) == WIFI_ERR_NONE)
      return VOC_ERR_NONE;
    else
      return VOC_ERR_CARRIER;
  }



/**Triggers cancelation of an outgoing call.
 *Triggers rejection of an incoming call by the local peer.
 *Triggers termination of a connected call by the local peer. 
 *
 *@param sid 	the Session ID
 *@param lid	the target Call ID
 *
 *@return 0 if successfull; otherwise a negative error code 
 */
  voc_err_t voc_wifi_hungup (voc_sid_t sid, voc_cid_t cid)
  {
    if (tel_wifi_call_hungup (sid, cid) == WIFI_ERR_NONE)
      return VOC_ERR_NONE;
    else
      return VOC_ERR_CARRIER;
  }


  voc_err_t voc_wifi_hungup_all (voc_sid_t sid, voc_cid_t cid)
  {
  }



  voc_err_t voc_wifi_hungup_active (voc_sid_t sid, voc_cid_t cid)
  {
  }



  voc_err_t voc_wifi_replace_hold (voc_sid_t sid, voc_cid_t cid)
  {
  }

  voc_err_t voc_wifi_hungup_outgoing (voc_sid_t sid, voc_cid_t cid)
  {
  }



  voc_err_t voc_wifi_hungup_held (voc_sid_t sid, voc_cid_t cid)
  {
  }



  voc_err_t voc_wifi_conference (voc_sid_t sid, voc_cid_t cid)
  {
  }


  voc_err_t voc_wifi_exit_conference (voc_sid_t sid, voc_cid_t cid)
  {
  }


#ifdef __cplusplus
}
#endif

/* 
vi:ts=2:nowrap:ai:expandtab
*/

